<?php

class XO_Event_Calendar {

	function __construct() {
		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
	}

	static function get_post_type() {
		return apply_filters( 'xo_event_calendar_event_type', XO_EVENT_CALENDAR_EVENT_POST_TYPE );
	}

	static function get_taxonomy_type() {
		return apply_filters( 'xo_event_calendar_event_type', XO_EVENT_CALENDAR_EVENT_TAXONOMY );
	}

	static function deactivation() {
		$uninstall = array(
			self::get_post_type(),
			self::get_taxonomy_type()
		);
		set_site_transient( 'xo_event_calendar_uninstall_options', $uninstall, MINUTE_IN_SECONDS );
	}

	public function activation() {
		$holiday_settings = get_option( 'xo_event_calendar_holiday_settings' );
		if ( $holiday_settings === false ) {
			$holiday_settings['day-off'] = array(
				'title' => __( 'Regular holiday', 'xo-event-calendar' ),
				'dayofweek' => array( 'sun' => true, 'mon' => false, 'tue' => false, 'wed' => false, 'thu' => false, 'fri' => false, 'sat' => true ),
				'special_holiday' => null,
				'non_holiday' => null,
				'color' => '#fcc0a7'
			);
			$holiday_settings['morning-off'] = array(
				'title' => __( 'Morning Off', 'xo-event-calendar' ),
				'dayofweek' => array( 'sun' => false, 'mon' => false, 'tue' => false, 'wed' => false, 'thu' => false, 'fri' => false, 'sat' => false ),
				'special_holiday' => null,
				'non_holiday' => null,
				'color' => '#ccebb9'
			);
			$holiday_settings['afternoon-off'] = array(
				'title' => __( 'Afternoon Off', 'xo-event-calendar' ),
				'dayofweek' => array( 'sun' => false, 'mon' => false, 'tue' => false, 'wed' => false, 'thu' => false, 'fri' => false, 'sat' => false ),
				'special_holiday' => null,
				'non_holiday' => null,
				'color' => '#bfcae2'
			);
			update_option( 'xo_event_calendar_holiday_settings', $holiday_settings );
		}

		// リライトルールをフラッシュするため
		$this->register_post_type();
		flush_rewrite_rules();
	}

	function plugins_loaded() {
		$ajax = 'xo_event_calendar_month';

		add_action( 'init',  array( $this, 'register_post_type' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		add_action( "wp_ajax_{$ajax}", array( $this, 'ajax' ) );
		add_action( "wp_ajax_nopriv_{$ajax}", array( $this, 'ajax' ) );
		add_shortcode('xo_event_calendar', array($this, 'event_calendar_shortcode'));
        add_action( 'widgets_init', array( $this, 'register_widget' ) );
		add_filter( 'template_include', array( $this, 'template_include' ) );
	}

	function enqueue_scripts() {
		wp_enqueue_style( 'dashicons' );
		wp_enqueue_style( 'xo-event-calendar', XO_EVENT_CALENDAR_URL . 'css/xo-event-calendar.css' );

		wp_enqueue_script( 'xo-event-calendar-ajax', XO_EVENT_CALENDAR_URL . 'js/ajax.js', array( 'jquery' ) );
		wp_localize_script( 'xo-event-calendar-ajax', 'xo_event_calendar_object', array(
			'ajax_url' => admin_url( 'admin-ajax.php' ),
			'action' => 'xo_event_calendar_month'
		) );
	}

	function ajax() {
		$id = isset( $_POST['id'] ) ? $_POST['id'] : '';
		$month = isset( $_POST['month'] ) ? $_POST['month'] : '';
		$event = isset( $_POST['event'] ) ? (bool)$_POST['event'] : false;
		$categories = isset( $_POST['categories'] ) ? $_POST['categories'] : '';
		$holidays = isset( $_POST['holidays'] ) ? $_POST['holidays'] : '';
		$prev = isset( $_POST['prev'] ) ? intval( $_POST['prev'] ) : -1;
		$next = isset( $_POST['next'] ) ? intval( $_POST['next'] ) : -1;
		preg_match( '/^([0-9]{4})-([0-9]{1,2})/', $month, $matches );
		if ( count( $matches ) === 3 ) {
			echo $this->get_calendar( $id, $matches[1], $matches[2], $event, $categories, $holidays, $prev, $next );
		}
		die();
	}

	function register_post_type() {
		$post_type = $this->get_post_type();
		$args = array(
			'labels' => array(
				'name' => _x( 'Events', 'post type general name', 'xo-event-calendar' ),
				'singular_name' => _x( 'Event', 'post type singular name', 'xo-event-calendar' ),
				'menu_name' => _x( 'Events', 'admin menu', 'xo-event-calendar' ),
				'name_admin_bar' => _x( 'Event', 'add new on admin bar', 'xo-event-calendar' ),
				'add_new' => _x( 'Add New', 'Event', 'xo-event-calendar' ),
				'add_new_item' => __( 'Add New Event', 'xo-event-calendar' ),
				'new_item' => __( 'New Event', 'xo-event-calendar' ),
				'edit_item' => __( 'Edit Event', 'xo-event-calendar' ),
				'view_item' => __( 'View Event', 'xo-event-calendar' ),
				'all_items' => __( 'All Events', 'xo-event-calendar' ),
				'search_items' => __( 'Search Events', 'xo-event-calendar' ),
				'parent_item_colon'  => __( 'Parent Events:', 'xo-event-calendar' ),
				'not_found' => __( 'No Events found.', 'xo-event-calendar' ),
				'not_found_in_trash' => __( 'No Events found in Trash.', 'xo-event-calendar' )
			),
			'public' => true,
			'menu_position' => 4,
			'menu_icon' => 'dashicons-calendar-alt',
			'supports' => array( 'title', 'editor', 'excerpt', 'thumbnail' ),
			'has_archive' => true
		);
		$args = apply_filters( 'xo_event_calendar_register_post_args', $args );
		register_post_type( $post_type, $args );
		
		$taxonomy_type = $this->get_taxonomy_type();
		$args = array(
			'labels' => array(
				'name' => _x( 'Categories', 'taxonomy general name', 'xo-event-calendar' ),
				'singular_name' => _x( 'Category', 'taxonomy singular name', 'xo-event-calendar' ),
				'search_items' => __( 'Search Categories', 'xo-event-calendar' ),
				'all_items' => __( 'All Categories', 'xo-event-calendar' ),
				'parent_item' => __( 'Parent Category', 'xo-event-calendar' ),
				'parent_item_colon' => __( 'Parent Category:', 'xo-event-calendar' ),
				'edit_item' => __( 'Edit Category', 'xo-event-calendar' ),
				'update_item' => __( 'Update Category', 'xo-event-calendar' ),
				'add_new_item' => __( 'Add New Category', 'xo-event-calendar' ),
				'new_item_name' => __( 'New Category Name', 'xo-event-calendar' ),
				'menu_name' => __( 'Category', 'xo-event-calendar' )
			),
			'hierarchical' => true,
			'show_ui' => true,
			'query_var' => true,
			'rewrite' => array( 'slug' => $taxonomy_type ),
		);
		$args = apply_filters( 'xo_event_calendar_register_taxonomy_args', $args );
		register_taxonomy( $taxonomy_type, $post_type, $args );
	}
	
	private function get_events( $start_date, $end_date, $terms = null ) {
		$post_type = $this->get_post_type();
		$taxonomy_type = $this->get_taxonomy_type();

		$args = array(
			'post_type' => $post_type,
			'post_status' => 'publish',
			'orderby' => array( 'event_start_date' => 'ASC', 'event_end_date' => 'DESC' ),
			'meta_query' => array(
				'relation' => 'AND',
				'event_start_date' => array ('key' => 'event_start_date', 'compare' => 'EXISTS' ),
				'event_end_date' => array( 'key' => 'event_end_date', 'compare' => 'EXISTS' ),
				array(
					'relation' => 'AND',
					array(
						array( 'key' => 'event_start_date', 'value' => date( 'Y-m-d', $end_date ), 'compare' => '<=', 'type' => 'DATE' ),
						array( 'key' => 'event_end_date', 'value' => date( 'Y-m-d', $start_date ), 'compare' => '>=', 'type' => 'DATE' ),
					)
				)
			),
			'paged' => get_query_var( 'paged' ),
			'posts_per_page' => -1
		);
		if ( !empty( $terms ) ) {
			$args['tax_query'] = array( array( 'taxonomy' => $taxonomy_type, 'field' => 'slug', 'terms' => $terms ) );
		}
		$query = new WP_Query( $args );

		$events = array();
		while ( $query->have_posts() ) {
			global $post;
			$query->the_post();

			// カテゴリーの色を取得
			$bg_color = '#ccc';
			$terms = get_the_terms( $post->ID, $taxonomy_type );
			if ( is_array( $terms ) ) {
				foreach ( $terms as $cate ) {
					$cat_data = get_option( 'xo_event_calendar_cat_' . intval( $cate->term_id ) );
					$cat_color = esc_html( $cat_data['category_color'] );
					if ( $cat_color ) {
						$bg_color = $cat_color;
						break;
					}
				}
			}

			$events[] = array(
				get_the_title(),
				get_post_meta( $post->ID, 'event_start_date', true ),
				get_post_meta( $post->ID, 'event_end_date', true ),
				$bg_color,
				get_permalink( $post->ID )
			);
		}
		wp_reset_postdata();

		return $events;
	}

	private function get_holiday_style( $holidays, $holiday_settings, $date ) {
		$weeks = array( 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' );
		$style = null;
		foreach ( $holidays as $holiday ) {
			if ( array_key_exists( $holiday, $holiday_settings ) ) {
				if ( ( $holiday_settings[$holiday]['dayofweek'][$weeks[date( 'w', $date )]] ||
					strpos( $holiday_settings[$holiday]['special_holiday'], date( 'Y-m-d', $date ) ) !== false ) &&
					strpos( $holiday_settings[$holiday]['non_holiday'], date( 'Y-m-d', $date ) ) === false
				) {
					$style = "background-color: {$holiday_settings[$holiday]['color']};";
				}
			}
		}
		return $style;
	}

	/**
	 * 月間カレンダーを取得します。
	 * 
	 * @param string $id カレンダーの ID。
	 * @param string $year 年。
	 * @param string $month 月。
	 * @param bool $event イベントを表示するかどうかを示す値。
	 * @param string $categories_string カテゴリーのスラッグ（カンマ区切り）。空の場合はすべてのカテゴリー。
	 * @param string $holidays_string 休日の名前（カンマ区切り）。
	 * @param integer $prev_month_feed 前月送り。-1 は制限なし。
	 * @param integer $next_month_feed 翌月送り。-1 は制限なし。
	 * @return string HTML
	 */
	private function get_calendar( $id, $year, $month, $show_event, $categories_string, $holidays_string, $prev_month_feed, $next_month_feed ) {
		global $wp_locale;

		// 月末の取得
		$last_day = date( 'j', mktime( 0, 0, 0, $month + 1, 0, $year ) );
		// 曜日の取得（0:日 ～ 6:土）
		$week = date( 'w', mktime( 0, 0, 0, $month, 1, $year ) );
		// 週数を取得
		$week_count = ceil( ( $week + $last_day ) / 7 );
		// 日数を取得
		$day_count = $week_count * 7;
		// 開始日を取得
		$start_date = strtotime( "{$year}-{$month}-1" );
		if ( $week > 0 ) {
			$start_date = strtotime( "-{$week} day", $start_date );
		}
		// 終了日を取得
		$end_date = strtotime( "+{$day_count} day", $start_date );
		// イベントを取得
		if ( $show_event ) {
			if ( empty( $categories_string ) ) {
				$events = $this->get_events( $start_date, $end_date );
			} else {
				$categories = explode( ',', $categories_string );
				$events = $this->get_events( $start_date, $end_date, $categories );
			}
			$event_js = '1';
		} else {
			$events = array();
			$event_js = '0';
		}

		$today = date_i18n( 'Y-m-d' );

		$holidays = explode( ',', $holidays_string );
		$holiday_settings = get_option( 'xo_event_calendar_holiday_settings' );
		
		// 月間カレンダー
		$month_html = '';
		$line_count = 0;
		for ( $week_index = 0; $week_index < $week_count; $week_index++ ) {
			$month_html .= '<tr><td colspan="7" class="month-week">';

			$month_html .= '<table class="month-dayname">';
			$month_html .= '<tbody>';
			$month_html .= '<tr class="dayname">';
			for ( $day_index = 0; $day_index < 7; $day_index++ ) {
				$d = $week_index * 7 + $day_index;
				$date = strtotime( "+{$d} days", $start_date );
				$class = '';
				if ( date( 'n', $date ) != $month ) $class .= 'other-month ';
				if ( date( 'Y-m-d', $date ) === $today ) $class .= 'today ';
				if ( !empty( $class ) ) $class = 'class="' . trim( $class ) . '" ';
				$style = $this->get_holiday_style( $holidays, $holiday_settings, $date );
				if ( !empty( $style ) ) $style = 'style="' . $style . '" ';
				$month_html .= "<td><div {$class}{$style}>" . date( 'j', $date ) . '</div></td>';
			}
			$month_html .= '</tr>';
			$month_html .= '</tbody>';
			$month_html .= '</table>';

			$month_html .= '<div class="month-dayname-space"></div>';

			if ( count( $events ) > 0 ) {
				$days_week =  array_fill( 0, 7, array() );
				for ( $day_index = 0; $day_index < 7; $day_index++ ) {
					$d = $week_index * 7 + $day_index;
					$day = date( 'Y-m-d', strtotime( "+{$d} day", $start_date ) );
					for ( $i = 0; $i < count( $events ); $i++ ) {
						$event = $events[$i];
						if ( empty( $event[2] ) ) {
							if ( $event[1] == $day ) {
								$days_week[$day_index][] = $i;
							}
						} else {
							if ( $event[1] <= $day && $day <= $event[2] ) {
								$days_week[$day_index][] = $i;
							}
						}
					}
				}

				for ( $day_index = 0; $day_index < 7; $day_index++ ) {
					for ( $line_index = 0; $line_index < count( $days_week[$day_index] ); $line_index++ ) {
						if ( isset( $days_week[$day_index][$line_index] ) && $days_week[$day_index][$line_index] != null ) {
							$max_line = -1;
							for ( $i = 0; $i < 7; $i++ ) {
								for ( $j = 0; $j < count( $days_week[$i] ); $j++ ) {
									if ( isset( $days_week[$i][$j] ) && $days_week[$i][$j] == $days_week[$day_index][$line_index] ) {
										if ( $max_line < $j ) $max_line = $j;
									}
								}
							}
							if ( $max_line > $line_index ) {
								array_splice( $days_week[$day_index], $line_index, 0, -1 );
								$days_week[$day_index][$line_index] = -1;
							}
						}
					}
				}

				for ( $day_index = 0; $day_index < 7; $day_index++ ) {
					for ( $line_index = 0; $line_index < count( $days_week[$day_index] ); $line_index++ ) {
						if ( isset( $days_week[$day_index][$line_index] ) && $days_week[$day_index][$line_index] === -1 ) {
							for ( $i = $line_index + 1; $i < count( $days_week[$day_index] ); $i++ ) {

								$index = $days_week[$day_index][$i];

								if ( isset( $days_week[$day_index][$i] ) && empty( $events[$index][2] ) ) {
									$days_week[$day_index][$line_index] = $days_week[$day_index][$i];
									unset( $days_week[$day_index][$i] );
								}
							}
						}
					}
				}

				$line_count = max(
					count( $days_week[0] ), count( $days_week[1] ), count( $days_week[2] ),
					count( $days_week[3] ),	count( $days_week[4] ), count( $days_week[5] ),
					count( $days_week[6] )
				);

				for ( $line_index = 0; $line_index < $line_count; $line_index++ ) {
					$month_html .= '<table class="month-event">';
					$month_html .= '<tbody>';
					$month_html .= '<tr>';
					for ( $day_index = 0; $day_index < 7; $day_index++ ) {
						if ( !isset( $days_week[$day_index][$line_index] ) ) {
							$month_html .= '<td></td>';
						} else if ( $days_week[$day_index][$line_index] === -1 ) {
							$month_html .= '<td></td>';
						} else {
							$colspan = 1;
							if ( $day_index < 7 ) {
								for ( $i = $day_index + 1; $i < 7; $i++ ) {
									if ( isset( $days_week[$i][$line_index] ) && $days_week[$day_index][$line_index] == $days_week[$i][$line_index] ) {
										$colspan++;
									} else {
										break;
									}
								}
							}
							if ( $colspan > 0 ) {
								$event_index = $days_week[$day_index][$line_index];
								$event_title = $events[$event_index][0];
								$color = $events[$event_index][3];
								$url = $events[$event_index][4];
								$hsv = XO_Color::getHsv( XO_Color::getRgb( $color ) );
								$font_color = $hsv['v'] > 0.8 ? '#333' : '#eee';
								$month_html .= "<td colspan=\"{$colspan}\">";
								$month_html .= "<a href=\"{$url}\" title=\"{$event_title}\">";
								$month_html .= "<span class=\"month-event-title\" style=\"color: {$font_color}; background-color: {$color};\">{$event_title}</span>";
								$month_html .= '</a>';
								$month_html .= '</td>';
								$day_index += $colspan - 1;
							}
						}
					}
					$month_html .= '</tr>';
					$month_html .= '</tbody>';
					$month_html .= '</table>';
				}
			}

			if ( $line_count === 0 ) {
				$month_html .= '<table class="month-event-space">';
				$month_html .= '<tbody><tr><td><div></div></td><td><div></div></td><td><div></div></td><td><div></div></td><td><div></div></td><td><div></div></td><td><div></div></td></tr></tbody>';
				$month_html .= '</table>';
			}

			$month_html .= '</td></tr>';
		}

		$m = strtotime( "{$year}-{$month}-1" );
		$prev_month = date( 'Y-n', strtotime( '-1 month', $m ) );
		$next_month = date( 'Y-n', strtotime( '+1 month', $m ) );
		$prev_text = '<span class="dashicons dashicons-arrow-left-alt2"></span>';
		$next_text = '<span class="dashicons dashicons-arrow-right-alt2"></span>';
		$calendar_caption = sprintf( _x( '%1$s %2$s', 'calendar caption', 'xo-event-calendar' ), $wp_locale->get_month( $month ), $year );

		$html =	'<table class="xo-month">';
		$html .= '<caption>';
		$html .= '<div class="month-header">';
		if ( $prev_month_feed === -1 || $m > strtotime( "-{$prev_month_feed} month", strtotime( date_i18n( 'Y-m-1' ) ) ) ) {
			$html .= "<button type=\"button\" class=\"month-prev\" onclick=\"this.disabled = true; xo_event_calendar_month('{$id}','{$prev_month}','{$event_js}','{$categories_string}','{$holidays_string}','{$prev_month_feed}','{$next_month_feed}'); return false;\">{$prev_text}</button>";
		} else {
			$html .= "<button type=\"button\" class=\"month-prev\" disabled=\"disabled\">{$prev_text}</button>";
		}
		$html .= "<span>{$calendar_caption}</span>";
		if ( $next_month_feed === -1 || $m < strtotime( "+{$next_month_feed} month", strtotime( date_i18n( 'Y-m-1' ) ) ) ) {
			$html .= "<button type=\"button\" class=\"month-next\" onclick=\"this.disabled = true; xo_event_calendar_month('{$id}','{$next_month}','{$event_js}','{$categories_string}','{$holidays_string}','{$prev_month_feed}','{$next_month_feed}'); return false;\">{$next_text}</button>";
		} else {
			$html .= "<button type=\"button\" class=\"month-next\" disabled=\"disabled\" >{$next_text}</button>";
		}
		$html .= '</div>';
		$html .= '</caption>';
		$html .= '<thead>';
		$html .= '<tr>';
		$html .= '<th class="sunday">' . __( 'Sun', 'xo-event-calendar' ) . '</th>';
		$html .= '<th>' . __( 'Mon', 'xo-event-calendar' ) . '</th>';
		$html .= '<th>' . __( 'Tue', 'xo-event-calendar' ) . '</th>';
		$html .= '<th>' . __( 'Wed', 'xo-event-calendar' ) . '</th>';
		$html .= '<th>' . __( 'Thu', 'xo-event-calendar' ) . '</th>';
		$html .= '<th>' . __( 'Fri', 'xo-event-calendar' ) . '</th>';
		$html .= '<th class="saturday">' . __( 'Sat', 'xo-event-calendar' ) . '</th>';
		$html .= '</tr>';
		$html .= '</thead>';
		$html .= '<tbody>';
		$html .= $month_html;
		$html .= '</tbody>';
		$html .= '</table>' . "\n";

		return $html;
	}

	/**
	 * イベント カレンダーを取得します。
	 */
	function get_event_calendar( $id, $show_event, $categories_string, $holidays_string, $prev_month_feed, $next_month_feed ) {
		$year = date_i18n( 'Y' );
		$month = date_i18n( 'n' );
		$holiday_settings = get_option( 'xo_event_calendar_holiday_settings' );
		$holidays = explode( ',', $holidays_string );
		
		$retour = "<div id=\"{$id}\" class=\"xo-event-calendar\" >";
		$retour .= $this->get_calendar( $id, $year, $month, $show_event, $categories_string, $holidays_string, $prev_month_feed, $next_month_feed );
		foreach ( $holidays as $holiday ) {
			if ( array_key_exists( $holiday, $holiday_settings ) ) {
				$retour .= "<p class=\"holiday-title\"><span style=\"background-color: {$holiday_settings[$holiday]['color']};\"></span>{$holiday_settings[$holiday]['title']}</p>";
			}
		}
		$retour .= '<div class="loading-animation"></div>';
		$retour .= "</div>\n";
		return $retour;
	}

	/**
	 * イベント カレンダー ショートコードです。
	 */
	function event_calendar_shortcode( $args ) {
		$args = shortcode_atts( array(
			'id' => 'xo-event-calendar-1',
			'event' => 'true',
			'categories' => null,
			'holidays' => null,
			'previous' => -1,
			'next' => -1
		), $args );
		return $this->get_event_calendar( esc_attr( $args['id'] ), (bool)( strtolower( $args['event'] ) == 'true' ), $args['categories'], $args['holidays'], intval( $args['previous'] ), intval( $args['next'] ) );
	}

	function register_widget() {
		register_widget( 'XO_Widget_Event_Calendar' );
	}
	
	function template_include( $template ) {
		$post_type = $this->get_post_type();
		if ( is_singular( $post_type ) ) {
			$file_name = "single-{$post_type}.php";
		}
		if ( isset( $file_name ) ) {
			$theme_file = locate_template( $file_name );
		}
		if ( isset( $theme_file ) && $theme_file ) {
			// テンプレートあり
		} elseif ( isset( $file_name ) && $file_name ) {
			// テンプレートなし
			add_filter( 'next_post_link', '__return_false' );
			add_filter( 'previous_post_link', '__return_false' );
			add_filter( 'the_content', array( $this, 'single_content' ) );
		}
		return $template;
	}

	function get_event_date( $post_id ) {
		$custom = get_post_custom( $post_id );

		$all_day = (bool)$custom['event_all_day'][0];
		$start_date = $custom['event_start_date'][0];
		$start_hour = $custom['event_start_hour'][0];
		$start_minute = $custom['event_start_minute'][0];
		$end_date = !empty( $custom['event_end_date'][0] ) ? $custom['event_end_date'][0] : $start_date;
		$end_hour = $custom['event_end_hour'][0];
		$end_minute = $custom['event_end_minute'][0];

		$date_format = _x( 'Y/n/j', 'event meta', 'xo-event-calendar' ); // get_option( 'date_format' );
		$time_format = get_option( 'time_format' );
		
		$date = date( $date_format, strtotime( $start_date ) );
		if ( !$all_day ) $date .= ' ' . date( $time_format, strtotime( $start_hour . ':' . $start_minute  ) );
		if ( $start_date !== $end_date ) {
			if ( date( 'Y', strtotime( $start_date ) ) == date( 'Y', strtotime( $end_date ) ) ) {
				$date_format = _x( 'n/j', 'event meta', 'xo-event-calendar' );
			}
			$date .= ' - ' . date( $date_format, strtotime( $end_date ) );
			if ( !$all_day ) $date .= ' ' . date( $time_format, strtotime( $end_hour . ':' . $end_minute  ) );
		} else {
			if ( !$all_day ) $date .= ' - ' . date( $time_format, strtotime( $end_hour . ':' . $end_minute  ) );
		}

		return $date;
	}

	function single_content( $content ) {
		$post_id = get_the_ID();
		$taxonomy = $this->get_taxonomy_type();

		$details  = '<div class="xo-event-meta-details">';
		$details .= '<div class="xo-event-meta">';
		$details .= '<span class="eo-event-date">' . esc_html__( 'Event date:', 'xo-event-calendar' ) . ' ' . $this->get_event_date( $post_id ) . '</span>';
		if ( get_the_terms( $post_id, $taxonomy ) ) {
			$details .= '<span class="eo-event-category">' . esc_html__( 'Categories:', 'xo-event-calendar' ) . ' ' . get_the_term_list( $post_id, $taxonomy, '', ', ', '' ) . '</span>';
		}
		$details .= '</div>';
		$details .= '</div>' . "\n";

		return $details . $content;
	}

}
